package com.usky.fire.controller.web;


import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.usky.common.core.bean.ApiResult;
import com.usky.common.core.bean.CommonPage;
import com.usky.fire.domain.AlarmDetector;
import com.usky.fire.service.AlarmDetectorService;
import com.usky.fire.service.vo.AlarmDetectorVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 独立式无线探测器告警 前端控制器
 * </p>
 *
 * @author JCB
 * @since 2022-08-08
 */
@RestController
@RequestMapping("/alarmDetector")
public class AlarmDetectorController {

    @Autowired
    private AlarmDetectorService alarmDetectorService;

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-列表查询
     *
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param pageNum        当前页
     * @param pageSize       每页条数
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */

    @GetMapping("alarmDetectorLowerList")
    public ApiResult<CommonPage<AlarmDetectorVo>> alarmDetectorLowerList(@RequestParam(value = "handlingStatus", required = false) Integer handlingStatus,
                                                                         @RequestParam(value = "systemType", required = false) String systemType,
                                                                         @RequestParam(value = "startDate", required = false) String startDate,
                                                                         @RequestParam(value = "endDate", required = false) String endDate,
                                                                         @RequestParam(value = "pageNum", required = false, defaultValue = "1") Integer pageNum,
                                                                         @RequestParam(value = "pageSize", required = false, defaultValue = "10") Integer pageSize) {
        return ApiResult.success(alarmDetectorService.alarmDetectorLowerList(handlingStatus, startDate, endDate, pageNum, pageSize, systemType));
    }

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-左侧数据统计
     *
     * @param systemType 系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    @GetMapping("detectorStatistics")
    public ApiResult<Map<String, Object>> detectorStatistics(@RequestParam(value = "systemType", required = false) String systemType) {
        return ApiResult.success(alarmDetectorService.detectorStatistics(systemType));
    }

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-数据统计图
     *
     * @param systemType 系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @param startDate  开始时间  格式：yyyy-MM-dd HH:mm:ss
     * @param endDate    结束时间  格式：yyyy-MM-dd HH:mm:ss
     * @return
     */
    @GetMapping("detectorStatisticalChart")
    public ApiResult<Map<String, Object>> detectorStatisticalChart(@RequestParam(value = "systemType", required = false) String systemType,
                                                                   @RequestParam(value = "startDate", required = false) String startDate,
                                                                   @RequestParam(value = "endDate", required = false) String endDate) {
        return ApiResult.success(alarmDetectorService.detectorStatisticalChart(systemType, startDate, endDate));
    }

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-告警处理
     *
     * @param alarmDetector
     * @return
     */
    @PostMapping("updateAlarmDetector")
    public ApiResult<Void> updateAlarmDetector(@RequestBody AlarmDetector alarmDetector) {
        alarmDetectorService.updateAlarmDetector(alarmDetector);
        return ApiResult.success();
    }

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-导出
     *
     * @param response
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param exportTitle    导出标题
     * @throws IOException
     */
    @GetMapping("alarmDetectorLowerListExport")
    public void export(HttpServletResponse response,
                       @RequestParam(value = "handlingStatus", required = false) Integer handlingStatus,
                       @RequestParam(value = "systemType", required = false) String systemType,
                       @RequestParam(value = "startDate", required = false) String startDate,
                       @RequestParam(value = "endDate", required = false) String endDate,
                       @RequestParam(value = "exportTitle") String exportTitle) throws IOException {
        List<AlarmDetectorVo> list = alarmDetectorService.alarmDetectorLowerListExport(handlingStatus, startDate, endDate, systemType);
        ExcelUtil<AlarmDetectorVo> util = new ExcelUtil<AlarmDetectorVo>(AlarmDetectorVo.class);
        util.exportExcel(response, list, exportTitle, exportTitle);
    }

}

