package com.usky.fire.service;

import com.usky.common.core.bean.CommonPage;
import com.usky.common.mybatis.core.CrudService;
import com.usky.fire.domain.AlarmDetector;
import com.usky.fire.service.vo.AlarmDetectorVo;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 独立式无线探测器告警 服务类
 * </p>
 *
 * @author JCB
 * @since 2022-08-08
 */
public interface AlarmDetectorService extends CrudService<AlarmDetector> {
    /**
     * 事件管理-隐患管理-自动喷水灭火系统-列表查询
     *
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param pageNum        当前页
     * @param pageSize       每页条数
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    CommonPage<AlarmDetectorVo> alarmDetectorLowerList(Integer handlingStatus, String startDate, String endDate,
                                                       Integer pageNum, Integer pageSize, String systemType);

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-左侧统计
     *
     * @param systemType 系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    Map<String, Object> detectorStatistics(String systemType);


    /**
     * 事件管理-隐患管理-自动喷水灭火系统-数据统计图
     *
     * @param systemType 系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @param startDate  开始时间  格式：yyyy-MM-dd HH:mm:ss
     * @param endDate    结束时间  格式：yyyy-MM-dd HH:mm:ss
     * @return
     */
    Map<String, Object> detectorStatisticalChart(String systemType, String startDate, String endDate);

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-告警处理
     *
     * @param alarmDetector
     * @return
     */
    void updateAlarmDetector(AlarmDetector alarmDetector);

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-导出数据查询
     *
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    List<AlarmDetectorVo> alarmDetectorLowerListExport(Integer handlingStatus, String startDate, String endDate, String systemType);

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-数量统计
     *
     * @param deviceCodeList 设备编号
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    Integer detectorCount(List<String> deviceCodeList, Integer handlingStatus, List<Integer> systemTypeList);

    /**
     * 事件管理-隐患管理-自动喷水灭火系统-列表查询一
     *
     * @param deviceCodeList 设备编号
     * @param handlingStatus 处理状态（0、未处理 1、已处理）
     * @param startDate      开始时间 格式：yyyy-MM-dd HH:mm:ss
     * @param endDate        结束时间 格式：yyyy-MM-dd HH:mm:ss
     * @param pageNum        当前页
     * @param pageSize       每页条数
     * @param systemType     系统类型（1.火灾自动报警系统 2.自动喷水灭火系统 3.消防给水及消火栓系统 4.防排烟系统 5.电气火灾系统 6.消防视频监控系统）
     * @return
     */
    CommonPage<AlarmDetector> alarDetectorList(List<String> deviceCodeList, Integer handlingStatus, String startDate,
                                               String endDate, Integer pageNum, Integer pageSize, List<Integer> systemTypeList);
}
