package com.usky.fire.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.fire.domain.PatrolInspectionArea;
import com.usky.fire.mapper.PatrolInspectionAreaMapper;
import com.usky.fire.service.PatrolInspectionAreaService;
import com.usky.fire.service.util.OnlineMethod;
import com.usky.fire.service.vo.PatrolInspectionAreaVo;
import com.usky.system.model.LoginUser;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author JCB
 * @since 2022-07-12
 */
@Service
public class PatrolInspectionAreaServiceImpl extends AbstractCrudService<PatrolInspectionAreaMapper, PatrolInspectionArea> implements PatrolInspectionAreaService {

    /**
     * 巡查自检-巡检区域-新增
     *
     * @param patrolInspectionAreaVo
     */
    @Override
    @Transactional
    public void addPatrolInspectionArea(PatrolInspectionAreaVo patrolInspectionAreaVo) {
        PatrolInspectionArea patrolInspectionArea = new PatrolInspectionArea();
        patrolInspectionArea.setAreaName(patrolInspectionAreaVo.getAreaName());
        patrolInspectionArea.setAreaFid(0);
        patrolInspectionArea.setCompanyId(patrolInspectionAreaVo.getCompanyId());
        patrolInspectionArea.setCreator(SecurityUtils.getUsername());
        patrolInspectionArea.setCreateTime(LocalDateTime.now());
        patrolInspectionArea.setEnable(1);
        patrolInspectionArea.setTenantId(SecurityUtils.getTenantId());
        this.save(patrolInspectionArea);
        Integer areaFid = patrolInspectionArea.getId();
        for (int i = 0; i < patrolInspectionAreaVo.getAreaNameArray().length; i++) {
            PatrolInspectionArea patrolInspectionAreaZ = new PatrolInspectionArea();
            patrolInspectionAreaZ.setAreaName(patrolInspectionAreaVo.getAreaNameArray()[i]);
            patrolInspectionAreaZ.setAreaFid(areaFid);
            patrolInspectionAreaZ.setCompanyId(patrolInspectionAreaVo.getCompanyId());
            patrolInspectionAreaZ.setCreator(SecurityUtils.getUsername());
            patrolInspectionAreaZ.setCreateTime(LocalDateTime.now());
            patrolInspectionAreaZ.setEnable(1);
            patrolInspectionAreaZ.setTenantId(SecurityUtils.getTenantId());
            this.save(patrolInspectionAreaZ);
        }
    }

    @Override
    public void addPatrolInspectionAreaSon(PatrolInspectionArea patrolInspectionArea) {
        LambdaQueryWrapper<PatrolInspectionArea> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PatrolInspectionArea::getEnable, 1)
                .eq(PatrolInspectionArea::getId, patrolInspectionArea.getAreaFid());
        List<PatrolInspectionArea> list = this.list(queryWrapper);
        if (list.size() <= 0) {
            throw new BusinessException("父级ID不存在");
        }
        patrolInspectionArea.setCompanyId(list.get(0).getCompanyId());
        patrolInspectionArea.setCreator(SecurityUtils.getUsername());
        patrolInspectionArea.setCreateTime(LocalDateTime.now());
        patrolInspectionArea.setEnable(1);
        patrolInspectionArea.setTenantId(SecurityUtils.getTenantId());
        this.save(patrolInspectionArea);
    }

    /**
     * 巡查自检-巡检区域-编辑
     *
     * @param patrolInspectionArea
     */
    @Override
    public void updatePatrolInspectionArea(PatrolInspectionArea patrolInspectionArea) {
        this.updateById(patrolInspectionArea);
    }

    /**
     * 巡查自检-巡检区域-删除
     *
     * @param id 区域ID
     */
    @Override
    @Transactional
    public void delPatrolInspectionArea(Integer id) {
        LambdaQueryWrapper<PatrolInspectionArea> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PatrolInspectionArea::getEnable, 1).eq(PatrolInspectionArea::getAreaFid, id);
        List<PatrolInspectionArea> patrolInspectionArealist = this.list(queryWrapper);
        if (null != patrolInspectionArealist && !patrolInspectionArealist.isEmpty()) {
            for (int i = 0; i < patrolInspectionArealist.size(); i++) {
                PatrolInspectionArea patrolInspectionAreaz = new PatrolInspectionArea();
                patrolInspectionAreaz.setId(patrolInspectionArealist.get(i).getId());
                patrolInspectionAreaz.setEnable(0);
                this.updateById(patrolInspectionAreaz);
            }
        }
        PatrolInspectionArea patrolInspectionArea = new PatrolInspectionArea();
        patrolInspectionArea.setId(id);
        patrolInspectionArea.setEnable(0);
        this.updateById(patrolInspectionArea);
    }

    /**
     * 巡查自检-巡检区域-列表查詢
     *
     * @param id       区域ID
     * @param areaName 区域名称
     * @return
     */
    @Override
    public List<PatrolInspectionArea> patrolInspectionAreaList(Integer id, String areaName) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String userType = null;
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        LambdaQueryWrapper<PatrolInspectionArea> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PatrolInspectionArea::getEnable, 1)
                .eq(PatrolInspectionArea::getTenantId, SecurityUtils.getTenantId());
        if ("00".equals(userType)) {
            queryWrapper.eq(PatrolInspectionArea::getCreator, SecurityUtils.getUsername());
        }
        if (id != null && id != 0) {
            queryWrapper.eq(PatrolInspectionArea::getId, id);
        }
        if (!"".equals(areaName) && areaName != null) {
            queryWrapper.like(PatrolInspectionArea::getAreaName, areaName);
        }
        queryWrapper.orderByDesc(PatrolInspectionArea::getId);
        List<PatrolInspectionArea> list = this.list(queryWrapper);
        return list;
    }


    /**
     * 巡查自检-巡检区域-一级区域下拉框
     *
     * @return
     */
    @Override
    public List<PatrolInspectionArea> patrolInspectionAreaSelect(Integer areaFid) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        String userType = null;
        if (loginUser != null && !"".equals(loginUser)) {
            userType = loginUser.getUserType();
        }
        LambdaQueryWrapper<PatrolInspectionArea> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(PatrolInspectionArea::getTenantId, SecurityUtils.getTenantId())
                .eq(PatrolInspectionArea::getEnable, 1)
                .eq(PatrolInspectionArea::getAreaFid, areaFid);
        if ("00".equals(userType)) {
            queryWrapper.eq(PatrolInspectionArea::getCreator, SecurityUtils.getUsername());
        }
        queryWrapper.orderByDesc(PatrolInspectionArea::getId);
        return this.list(queryWrapper);
    }
}
